@@ -267,17 +267,39 @@ else {
     $bal->event($_ => 'down') foreach @down;
 }
 
-syslog('info',"adjusting routing tables...");
+if ($bal->operating_mode eq 'failover') {
+    my $s = $bal->preferred_service;
+    syslog('info',"Setting default route to service %s, device %s",$s,$bal->dev($s));
+} else {
+    syslog('info',"Adjusting routing tables.");
+}
 $bal->set_routes_and_firewall();
-start_lsm($bal) unless @ARGV || $DEBUG;
+kill_lsm() unless @ARGV;
+start_or_reload_lsm($bal);
 
 exit 0;
 
 sub do_status {
     my $state = $bal->event();
-    for my $svc (sort $bal->isp_services) {
-	printf("%-15s %8s\n",$svc,$state->{$svc}||'unknown');
+    my @svc = sort $bal->isp_services;
+    printf("%-12s %-8s %-8s %-3s\n",
+	   'Service',
+	   'Device',
+	   'State',
+	   'Routing');
+    foreach (@svc) {
+	my $preferred = $bal->preferred_service;
+	my $routing   = $bal->operating_mode eq 'failover' ? $_ eq $preferred
+	                                                   : $state->{$_} eq 'up';
+	printf("%-12s %-8s %-8s %-3s\n",
+		   $_,
+		   $bal->dev($_),
+		   $state->{$_}||'unknown',
+		   $routing ? 'yes' : 'no',
+
+	    );
     }
+
     if ($< == 0)  { # running as root
 	kill(USR1 => `cat /var/run/lsm.pid`);
 	print STDERR "See syslog for detailed link monitoring information from lsm.\n";
@@ -286,33 +308,60 @@ sub do_status {
 }
 
 sub do_kill_lsm {
+    kill_lsm();
+    exit 0;
+}
+
+sub kill_lsm {
     my $lsm_running = -e '/var/run/lsm.pid' && kill(0=>`cat /var/run/lsm.pid`);
     if ($lsm_running) {
 	kill(TERM => `cat /var/run/lsm.pid`);
 	print STDERR "lsm process killed\n";
-    } else {
-	print STDERR "lsm does not seem to be running\n";
     }
-    exit 0;
 }
 
-sub start_lsm {
+sub start_or_reload_lsm {
     my $bal = shift;
 
-    my $lsm_conf = $bal->lsm_conf_file;
-    my $bal_conf = $bal->bal_conf_file;
+    my $config_changed = write_lsm_config($bal);
+    my $lsm_conf       = $bal->lsm_conf_file;
+    my $lsm_pid        = -e '/var/run/lsm.pid' && `cat /var/run/lsm.pid`;
+    chomp($lsm_pid);
 
-    # kill existing lsm
-    $bal->signal_lsm('TERM');
+    my $lsm_running = $lsm_pid && kill(0=>$lsm_pid);
 
+    if (!$lsm_running) {
+	print STDERR  "Starting lsm link status monitoring daemon\n";    
+	syslog('info',"Starting lsm link status monitoring daemon");    
+	$bal->start_lsm();
+    }
+    elsif ($ARGV[0] && $ARGV[0] eq 'long_down') {
+	print STDERR  "Reloading lsm link status monitoring daemon\n";    
+	syslog('info',"Reloading lsm link status monitoring daemon");    
+	kill(HUP => $lsm_pid);
+    }
+    
+}
+
+sub write_lsm_config {
+    my $bal = shift;
+
+    my $lsm_conf = $bal->lsm_conf_file();
+
+    my $old_text = '';
+    open my $fh,$lsm_conf or die "$lsm_conf: $!";
+    $old_text .= $_ while <$fh>;
+    close $fh;
+
+    my $new_text = $bal->lsm_config_text();
+    return if $new_text eq $old_text;
+    
     # Create config file
-    open my $fh,'>',$lsm_conf or die "$lsm_conf: $!";
-    print $fh $bal->lsm_config_text();
+    open $fh,'>',$lsm_conf or die "$lsm_conf: $!";
+    print $fh $new_text;
     close $fh or die "$lsm_conf: $!";
-
-    # now start the process
-    syslog('info',"Starting lsm link status monitoring daemon");    
-    $bal->start_lsm();
+    
+    return 1;
 }
 
 sub fatal_error {
