@@ -4,11 +4,12 @@ use strict;
 use Fcntl ':flock';
 use Carp 'croak','carp';
 use Data::Dumper;
+no warnings;
 
 eval 'use Net::Netmask';
 eval 'use Net::ISP::Balance::ConfigData';
 
-our $VERSION    = '1.20';
+our $VERSION    = '1.22';
 
 =head1 NAME
 
@@ -442,6 +443,23 @@ sub echo_only {
     $d;
 }
 
+=head2 $mode = $bal->operating_mode([$mode])
+
+Set or interrogate the operating mode. Will return one of "balanced"
+(currently the default) or "failover". This corresponds to the "mode"
+option in the configuration file. If the option is neither "balanced"
+nor "failover", then "balanced" is chosen (be warned!)
+
+=cut
+
+sub operating_mode {
+    my $self = shift;
+    my $d    = $self->{operating_mode};
+    $self->{operating_mode} = shift if @_;
+    return 'failover' if $d && $d =~ /failover/i;
+    return 'balanced';
+}
+
 =head2 $retries = $bal->dev_lookup_retries([$retries])
 
 Get/set the number of times the library will try to look up an interface
@@ -938,7 +956,7 @@ Return list of service names that correspond to load-balanced ISPs.
 sub isp_services {
     my $self = shift;
     my @n    = $self->service_names;
-    return grep {$self->role($_) eq 'isp'} @n;
+    return grep {$self->role($_) eq 'isp'} @n; # kill uninit warning
 }
 
 =head2 @names = $bal->lan_services
@@ -951,7 +969,7 @@ Return list of service names that correspond to lans.
 sub lan_services {
     my $self = shift;
     my @n    = $self->service_names;
-    return grep {$self->role($_) eq 'lan'} @n;
+    return grep {$self->role($_) eq 'lan'} @n; # kill uninit warning...
 }
 
 =head2 $state = $bal->event($service => $new_state)
@@ -1360,12 +1378,12 @@ sub lsm_config_text {
                       -eventscript            => $balance_script,
                       -long_down_eventscript  => $balance_script,
                       -notifyscript         => "$scripts_dir/default_script",
-                      -max_packet_loss      => 15,
+                      -max_packet_loss          => 20,
                       -max_successive_pkts_lost =>  7,
-                      -min_packet_loss          =>  5,
-                      -min_successive_pkts_rcvd =>  10,
+                      -min_packet_loss          =>  10,
+                      -min_successive_pkts_rcvd =>  5,
                       -interval_ms              => 1000,
-                      -timeout_ms               => 1000,
+                      -timeout_ms               => 500,
 	              -long_down_time           => 120,
                       -warn_email               => 'root',
                       -check_arp                =>  0,
@@ -1418,6 +1436,10 @@ sub _parse_configuration_file {
 	    push @forwarding_group,\@group;
 	    next;
 	}
+	if (/^mode\s*=\s*(.+)$/) {   # operating mode
+	    $self->operating_mode($1);
+	    next;
+	}
 	if (/^(\w+)\s*=\s*(.*)$/) { # lsm config
 	    $lsm_options{"-${1}"} = $2;
 	    next;
@@ -1680,8 +1702,7 @@ sub set_firewall {
     } else {
 	$self->base_fw_rules();
     }
-
-    $self->balancing_fw_rules();
+    $self->balancing_fw_rules();  # WARNING: This is a null-op in "failover" mode
     $self->sanity_fw_rules();
     $self->nat_fw_rules();
     $self->local_fw_rules();
@@ -1699,8 +1720,8 @@ sub enable_forwarding {
 }
 =head2 $bal->routing_rules()
 
-This method is called by set_routes() to emit the rules
-needed to create the routing rules.
+This method is called by set_routes() to emit the rules needed to
+create the routing rules.
 
 =cut
 
@@ -1708,7 +1729,16 @@ sub routing_rules {
     my $self = shift;
     # main table
     $self->ip_route("add ",$self->net($_),'dev',$self->dev($_),'src',$self->ip($_)) foreach $self->service_names;
-    $self->_create_default_route();
+
+    # different handling of the default route depending on whether we are in
+    # "balanced" or "failover" mode.
+    my $mode = $self->operating_mode;
+    if ($mode eq 'balanced') {
+	$self->_create_default_multipath_route();
+    } elsif ($mode eq 'failover') {
+	$self->_create_default_failover_route();
+    }
+
     $self->_create_service_routing_tables();
 }
 
@@ -1725,9 +1755,8 @@ END
     $self->ip_route("flush table ",$self->table($_)) foreach $self->isp_services;
 }
 
-sub _create_default_route {
+sub _create_default_multipath_route {
     my $self = shift;
-    my $D    = shift;
 
     my @up = $self->up;
 
@@ -1755,6 +1784,26 @@ sub _create_default_route {
 
 }
 
+sub _create_default_failover_route {
+    my $self = shift;
+    my $preferred = $self->preferred_service;
+    print STDERR "# setting single default route via $preferred\n" if $self->verbose;
+    $self->ip_route("add default via",$self->gw($preferred),'dev',$self->dev($preferred));
+}
+
+=head2 $service = $bal->preferred_service
+
+Returns the preferred service, which is the currently running service with the highest weight. Used for
+failover mode.
+
+=cut
+
+sub preferred_service {
+    my $self = shift;
+    my @up = sort { $self->weight($b) <=> $self->weight($a) } $self->up;
+    return $up[0];
+}
+
 sub _create_service_routing_tables {
     my $self = shift;
 
@@ -1942,6 +1991,8 @@ rules for balancing outgoing connections.
 sub balancing_fw_rules {
     my $self = shift;
 
+    return unless $self->operating_mode eq 'balanced';
+
     print STDERR "# balancing FW rules\n" if $self->verbose;
 
     for my $svc ($self->isp_services) {
